<?php
/**
 *
 * @package phpBB Extension - Manage Attachments
 * @copyright (c) 2018 Picaron
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 *
 */

namespace pikaron\manageattachments\acp;

/**
* @package acp
*/
class acp_manageattachments_module
{
    function main($id, $mode)
    {
        global $db, $user, $auth, $template, $cache, $config, $phpbb_root_path, $request, $phpEx, $phpbb_container, $phpbb_log;

        $notify = array();
        $submit = $request->variable('submit', false);

        if ($submit)
        {
            /// Delete attachments
            $delete_files      = (isset($_POST['delete'])) ? array_keys($request->variable('delete', array('' => 0))) : array();

            /// Reassign attachments
            $add_files         = (isset($_POST['add'])) ? array_keys($request->variable('add', array('' => 0))) : array();
            $post_ids          = $request->variable('post_id', array('' => 0), true);
            $current_post_ids  = $request->variable('current_post_id', array('' => 0), true);
            $next_post_ids     = $post_ids;
            $current_topic_ids = $request->variable('current_topic_id', array('' => 0), true);

            // Function to avoid the reassignment of attachments marked to erase
            // If there are attachments to be deleted and to be reassigned, delete those marked to be deleted from the two variables
            if (sizeof($delete_files) && sizeof($add_files))
            {
                $add_files = array_diff($add_files, $delete_files);

                foreach ($delete_files as $attach_id)
                {
                    if (!empty($current_post_ids[$attach_id]) && !empty($next_post_ids[$attach_id]))
                    {
                        unset($current_post_ids[$attach_id]);
                        unset($next_post_ids[$attach_id]);
                    }
                }
            }

            // Mark to remove attachments in post and topics
            // Not applicable if there are no attachments to reassign or have all marked to be deleted
            $all_attach_post_ids = $pre_unset_post_ids = $pre_unset_topic_ids = $all_attach_topic_ids = $unset_topic_ids = $unset_post_ids = array();

            if (sizeof($add_files) && sizeof($current_post_ids) && sizeof($next_post_ids))
            {
                $pre_unset_post_ids = array_diff($current_post_ids, $next_post_ids);

                if (sizeof($pre_unset_post_ids))
                {
                    // We get all the post_msg_id of the attachments that are in each message
                    $sql = 'SELECT post_msg_id
                        FROM ' . ATTACHMENTS_TABLE . '
                        WHERE ' . $db->sql_in_set('post_msg_id', $pre_unset_post_ids);
                    $result = $db->sql_query($sql);

                    while ($row = $db->sql_fetchrow($result))
                    {
                        $all_attach_post_ids[] = $row['post_msg_id'];
                    }
                    $db->sql_freeresult($result);

                    $val_unset_post_ids = array_count_values($pre_unset_post_ids);
                    $val_all_attach_post_ids = array_count_values($all_attach_post_ids);

                    foreach ($val_unset_post_ids as $key => $val)
                    {
                        if ($val_unset_post_ids[$key] == $val_all_attach_post_ids[$key])
                        {
                            $unset_post_ids[$key] = $val;
                        }
                    }
                }
                else
                {
                    $unset_post_ids = $pre_unset_post_ids;
                }

                // We get all the topics from the posts
                $sql = 'SELECT topic_id
                    FROM ' . POSTS_TABLE . '
                    WHERE ' . $db->sql_in_set('post_id', $next_post_ids);
                $result = $db->sql_query($sql);
                $next_topic_ids = array();
                while ($row = $db->sql_fetchrow($result))
                {
                    $next_topic_ids[] = $row['topic_id'];
                }
                $db->sql_freeresult($result);

                $pre_unset_topic_ids = array_diff($current_topic_ids, $next_topic_ids);

                // We get all the topic_id of the attachments that are in each message
                if (sizeof($pre_unset_topic_ids))
                {
                    $sql = 'SELECT topic_id
                        FROM ' . ATTACHMENTS_TABLE . '
                        WHERE ' . $db->sql_in_set('topic_id', $pre_unset_topic_ids);
                    $result = $db->sql_query($sql);

                    while ($row = $db->sql_fetchrow($result))
                    {
                        $all_attach_topic_ids[] = $row['topic_id'];
                    }
                    $db->sql_freeresult($result);

                    $val_unset_topic_ids = array_count_values($pre_unset_topic_ids);
                    $val_all_attach_topic_ids = array_count_values($all_attach_topic_ids);

                    foreach ($val_unset_topic_ids as $key => $val)
                    {
                        if ($val_unset_topic_ids[$key] == $val_all_attach_topic_ids[$key])
                        {
                            $unset_topic_ids[$key] = $val;
                        }
                    }
                }
                else
                {
                    $unset_topic_ids = $pre_unset_topic_ids;
                }
            }

            // We delete the selected attachments
            if (sizeof($delete_files))
            {
                // Select those attachments we want to delete...
                $sql = 'SELECT real_filename
                    FROM ' . ATTACHMENTS_TABLE . '
                    WHERE ' . $db->sql_in_set('attach_id', $delete_files) . '
                        AND is_orphan = 0';
                $result = $db->sql_query($sql);
                while ($row = $db->sql_fetchrow($result))
                {
                    $deleted_filenames[] = $row['real_filename'];
                }
                $db->sql_freeresult($result);
                delete_attachments('attach', $delete_files);
                $phpbb_log->add('admin', $user->data['user_id'], $user->ip, 'LOG_ATTACH_DEL', false, array(implode(', ', $deleted_filenames)));
                $notify[] = sprintf($user->lang['LOG_ATTACH_DEL'], implode($user->lang['COMMA_SEPARATOR'], $deleted_filenames));
            }

            // We debug the list of attachments to be reassigned
            $upload_list = array();
            foreach ($add_files as $attach_id)
            {
                if (!in_array($attach_id, array_keys($delete_files)) && !empty($post_ids[$attach_id]) && $post_ids[$attach_id] != $current_post_ids[$attach_id])
                {
                    $upload_list[$attach_id] = $post_ids[$attach_id];
                }
            }
            unset($add_files);

            // We reassign the selected attachments
            if (sizeof($upload_list))
            {
                $template->assign_var('S_UPLOADING_FILES', true);

                $sql = 'SELECT forum_id, forum_name
                    FROM ' . FORUMS_TABLE;
                $result = $db->sql_query($sql, 86400);

                $forum_names = array();
                while ($row = $db->sql_fetchrow($result))
                {
                    $forum_names[$row['forum_id']] = $row['forum_name'];
                }
                $db->sql_freeresult($result);

                $sql = 'SELECT forum_id, topic_id, post_id, poster_id, post_subject
                    FROM ' . POSTS_TABLE . '
                    WHERE ' . $db->sql_in_set('post_id', $upload_list);
                $result = $db->sql_query($sql);

                $post_info = array();
                while ($row = $db->sql_fetchrow($result))
                {
                    $post_info[$row['post_id']] = $row;
                }
                $db->sql_freeresult($result);

                // Select those attachments we want to change...
                $sql = 'SELECT *
                    FROM ' . ATTACHMENTS_TABLE . '
                    WHERE ' . $db->sql_in_set('attach_id', array_keys($upload_list)) . '
                        AND is_orphan = 0';
                $result = $db->sql_query($sql);

                while ($row = $db->sql_fetchrow($result))
                {
                    if (isset($post_info[$upload_list[$row['attach_id']]]))
                    {
                        $post_row = $post_info[$upload_list[$row['attach_id']]];
                        $template->assign_block_vars('upload', array(
                            'FILE_INFO'     => sprintf($user->lang['UPLOADING_FILE_TO'], $row['real_filename'], $forum_names[$post_row['forum_id']], $post_row['post_subject'], $post_row['post_id']),
                            'S_DENIED'      => (!$auth->acl_get('f_attach', $post_row['forum_id'])) ? true : false,
                            'DENIED'        => (!$auth->acl_get('f_attach', $post_row['forum_id'])) ? sprintf($user->lang['UPLOAD_DENIED_FORUM'], $forum_names[$post_row['forum_id']]) : '')
                        );
                    }
                    else
                    {
                        $template->assign_block_vars('upload', array(
                            'FILE_INFO'     => sprintf($user->lang['UPLOADING_FILE_TO'], $row['real_filename'], '', '', $upload_list[$row['attach_id']]),
                            'S_DENIED'      => true,
                            'DENIED'        => sprintf($user->lang['UPLOAD_TOPIC_UNKNOWN'], $upload_list[$row['attach_id']]))
                        );
                        $phpbb_log->add('admin', $user->data['user_id'], $user->ip, 'LOG_ATTACH_ERROR_REASSIGNED', false, array($upload_list[$row['attach_id']], $row['real_filename']));
                        continue;
                    }

                    if (!$auth->acl_get('f_attach', $post_row['forum_id']))
                    {
                        $phpbb_log->add('admin', $user->data['user_id'], $user->ip, 'LOG_ATTACH_ERROR_REASSIGNED', false, array($post_row['post_id'], $row['real_filename']));
                        continue;
                    }

                    // Adjust attachment entry
                    $sql_ary = array(
                        'in_message'    => 0,
                        'is_orphan'     => 0,
                        'poster_id'     => $post_row['poster_id'],
                        'post_msg_id'   => $post_row['post_id'],
                        'topic_id'      => $post_row['topic_id'],
                    );

                    $sql = 'UPDATE ' . ATTACHMENTS_TABLE . '
                        SET ' . $db->sql_build_array('UPDATE', $sql_ary) . '
                        WHERE attach_id = ' . $row['attach_id'];
                    $db->sql_query($sql);

                    $sql = 'UPDATE ' . POSTS_TABLE . '
                        SET post_attachment = 1
                        WHERE post_id = ' . $post_row['post_id'];
                    $db->sql_query($sql);

                    $sql = 'UPDATE ' . TOPICS_TABLE . '
                        SET topic_attachment = 1
                        WHERE topic_id = ' . $post_row['topic_id'];
                    $db->sql_query($sql);

                    $phpbb_log->add('admin', $user->data['user_id'], $user->ip, 'LOG_ATTACH_REASSIGNED', false, array($post_row['post_id'], $row['real_filename']));
                }
                $db->sql_freeresult($result);

                if (sizeof($unset_post_ids))
                {
                    $sql = 'UPDATE ' . POSTS_TABLE . '
                        SET post_attachment = 0
                        WHERE ' . $db->sql_in_set('post_id', $unset_post_ids);
                    $db->sql_query($sql);
                }

                if (sizeof($unset_topic_ids))
                {
                    $sql = 'UPDATE ' . TOPICS_TABLE . '
                        SET topic_attachment = 0
                        WHERE ' . $db->sql_in_set('topic_id', $unset_topic_ids);
                    $db->sql_query($sql);
                }
            }
        }

        // Pagination
        $start     = $request->variable('start', 0);

        // Sort keys
        $sort_days = $request->variable('st', 0);
        $sort_key  = $request->variable('sk', 't');
        $sort_dir  = $request->variable('sd', 'd');

        $form_key = 'acp_attach';
        add_form_key($form_key);

        if ($submit && !check_form_key($form_key))
        {
            trigger_error($user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
        }

        $this->tpl_name = 'acp_manageattachments';
        $this->page_title = 'ACP_MANAGE_ATTACHMENTS';

        // Sorting
        $limit_days = array(0 => $user->lang['ALL_ENTRIES'], 1 => $user->lang['1_DAY'], 7 => $user->lang['7_DAYS'], 14 => $user->lang['2_WEEKS'], 30 => $user->lang['1_MONTH'], 90 => $user->lang['3_MONTHS'], 180 => $user->lang['6_MONTHS'], 365 => $user->lang['1_YEAR']);
        $sort_by_text = array('f' => $user->lang['FILENAME'], 't' => $user->lang['FILEDATE'], 's' => $user->lang['FILESIZE'], 'x' => $user->lang['EXTENSION'], 'd' => $user->lang['DOWNLOADS'],'p' => $user->lang['ATTACH_POST_ID']);
        $sort_by_sql = array('f' => 'a.real_filename', 't' => 'a.filetime', 's' => 'a.filesize', 'x' => 'a.extension', 'd' => 'a.download_count', 'p' => 'a.post_msg_id');

        $s_limit_days = $s_sort_key = $s_sort_dir = $u_sort_param = '';
        gen_sort_selects($limit_days, $sort_by_text, $sort_days, $sort_key, $sort_dir, $s_limit_days, $s_sort_key, $s_sort_dir, $u_sort_param);

        $min_filetime = ($sort_days) ? (time() - ($sort_days * 86400)) : '';
        $limit_filetime = ($min_filetime) ? " AND a.filetime >= $min_filetime" : '';
        $start = ($sort_days && isset($_POST['sort'])) ? 0 : $start;

        $attachments_per_page = (int) $config['topics_per_page'];

        $sql = 'SELECT COUNT(a.attach_id) AS num_files, SUM(a.filesize) AS upload_dir_size
            FROM ' . ATTACHMENTS_TABLE . " a
            WHERE a.is_orphan = 0
                $limit_filetime";
        $result = $db->sql_query($sql);
        $row = $db->sql_fetchrow($result);
        $db->sql_freeresult($result);
        $num_files = (int) $row['num_files'];
        $total_size = (float) $row['upload_dir_size'];

        // Make sure $start is set to the last page if it exceeds the amount
        $pagination = $phpbb_container->get('pagination');
        $start = $pagination->validate_start($start, $attachments_per_page, $num_files);

        // If the user is trying to reach the second half of the attachments list, fetch it starting from the end
        $store_reverse = false;
        $sql_limit = $attachments_per_page;

        if ($start > $num_files / 2)
        {
            $store_reverse = true;

            // Select the sort order. Add time sort anchor for non-time sorting cases
            $sql_sort_anchor = ($sort_key != 't') ? ', a.filetime ' . (($sort_dir == 'd') ? 'ASC' : 'DESC') : '';
            $sql_sort_order = $sort_by_sql[$sort_key] . ' ' . (($sort_dir == 'd') ? 'ASC' : 'DESC') . $sql_sort_anchor;
            $sql_limit = $pagination->reverse_limit($start, $sql_limit, $num_files);
            $sql_start = $pagination->reverse_start($start, $sql_limit, $num_files);
        }
        else
        {
            // Select the sort order. Add time sort anchor for non-time sorting cases
            $sql_sort_anchor = ($sort_key != 't') ? ', a.filetime ' . (($sort_dir == 'd') ? 'DESC' : 'ASC') : '';
            $sql_sort_order = $sort_by_sql[$sort_key] . ' ' . (($sort_dir == 'd') ? 'DESC' : 'ASC') . $sql_sort_anchor;
            $sql_start = $start;
        }

        unset($attachments_list);
        $attachments_list = array();

        // Just get the files
        $sql = 'SELECT a.*, u.username, u.user_colour, t.topic_title, p.post_subject
            FROM ' . ATTACHMENTS_TABLE . ' a
            LEFT JOIN ' . USERS_TABLE . ' u ON (u.user_id = a.poster_id)
            LEFT JOIN ' . POSTS_TABLE . ' p ON (p.post_id = a.post_msg_id)
            LEFT JOIN ' . TOPICS_TABLE .
            " t ON (a.topic_id = t.topic_id AND a.in_message = 0)
            WHERE a.is_orphan = 0
                $limit_filetime
            ORDER BY $sql_sort_order";
        $result = $db->sql_query_limit($sql, $sql_limit, $sql_start);

        $i = ($store_reverse) ? $sql_limit - 1 : 0;

        // Store increment value in a variable to save some conditional calls
        $i_increment = ($store_reverse) ? -1 : 1;
        while ($attachment_row = $db->sql_fetchrow($result))
        {
            $attachments_list[$i] = $attachment_row;
            $i = $i + $i_increment;
        }
        $db->sql_freeresult($result);

        $base_url = $this->u_action . "&amp;$u_sort_param";
        $pagination->generate_template_pagination($base_url, 'pagination', 'start', $num_files, $attachments_per_page, $start);

        $template->assign_vars(array(
            'MANAGE_ATTACHMENTS_VERSION' => $config['version_manageattachments'],
            'U_ACTION'                   => $base_url."&amp;start=$start",
            'TOTAL_FILES'                => $num_files,
            'TOTAL_SIZE'                 => get_formatted_filesize($total_size),
            'S_LIMIT_DAYS'               => $s_limit_days,
            'S_SORT_KEY'                 => $s_sort_key,
            'S_SORT_DIR'                 => $s_sort_dir)
        );

        // Grab extensions
        $extensions = array();
        $extensions = $cache->obtain_attach_extensions(true);

        for ($i = 0, $end = sizeof($attachments_list); $i < $end; ++$i)
        {
            $row = $attachments_list[$i];

            $row['extension'] = strtolower(trim((string) $row['extension']));
            $comment = ($row['attach_comment'] && !$row['in_message']) ? str_replace(array("\n", "\r"), array('<br />', "\n"), $row['attach_comment']) : '';
            $display_cat = isset($extensions[$row['extension']]['display_cat']) ? $extensions[$row['extension']]['display_cat'] : ATTACHMENT_CATEGORY_NONE;
            $l_downloaded_viewed = ($display_cat == ATTACHMENT_CATEGORY_NONE) ? 'DOWNLOAD_COUNTS' : 'VIEWED_COUNTS';

            $template->assign_block_vars('attachments', array(
                'ATTACHMENT_POSTER' => get_username_string('full', (int) $row['poster_id'], (string) $row['username'], (string) $row['user_colour'], (string) $row['username']),
                'FILESIZE'          => get_formatted_filesize((int) $row['filesize']),
                'FILETIME'          => $user->format_date((int) $row['filetime']),
                'REAL_FILENAME'     => utf8_basename((string) $row['real_filename']),
                'COMMENT'           => $comment,
                'POST_TITLE'        => (!$row['in_message']) ? (string) $row['post_subject'] : '',
                'DISABLED'          => ($row['in_message']) ? 'disabled="disabled"' : '',
                'ATTACH_ID'         => (int) $row['attach_id'],
                'POST_ID'           => (int) $row['post_msg_id'],
                'TOPIC_ID'          => (int) $row['topic_id'],
                'POST_IDS'          => (!empty($post_ids[$row['attach_id']])) ? (int) $post_ids[$row['attach_id']] : '',
                'L_DOWNLOAD_COUNT'  => $user->lang($l_downloaded_viewed, (int) $row['download_count']),
                'S_IN_MESSAGE'      => (bool) $row['in_message'],
                'U_VIEW_POST'       => append_sid("{$phpbb_root_path}viewtopic.$phpEx", "t={$row['topic_id']}&amp;p={$row['post_msg_id']}") . "#p{$row['post_msg_id']}",
                'U_FILE'            => append_sid($phpbb_root_path . 'download/file.' . $phpEx, 'mode=view&amp;id=' . $row['attach_id']))
            );
        }

        if (sizeof($notify))
        {
            $template->assign_vars(array(
                'S_NOTIFY'      => true,
                'NOTIFY_MSG'    => implode('<br />', $notify))
            );
        }
    }
}
?>