<?php
/**
 *
 * @package phpBB Extension - Manage Attachments
 * @copyright (c) 2018 Picaron
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 *
 */

namespace pikaron\manageattachments\migrations;

class release_1_0_0 extends \phpbb\db\migration\migration
{

    public function effectively_installed()
    {
        return !empty($this->config['version_manageattachments']);
    }

    /**
    * Assign migration file dependencies for this migration
    *
    * @return array Array of migration files
    * @static
    * @access public
    */
    static public function depends_on()
    {
        return array('\phpbb\db\migration\data\v320\v320');
    }

    /**
    * Add Manage Attachments data to the database.
    *
    * @return array Array of table data
    * @access public
    */
    public function update_data()
    {
        return array(
            array('config.add', array('version_manageattachments', '1.0.0')),

            // Add ACP extension category
            array('module.add', array(
                'acp',
                'ACP_CAT_DOT_MODS',
                'ACP_MANAGE_ATTACHMENTS',
            )),

            // Add ACP preferences module
            array('module.add', array(
                'acp',
                'ACP_MANAGE_ATTACHMENTS',
                array(
                    'module_basename'   => '\pikaron\manageattachments\acp\acp_manageattachments_module',
                    'modes'             => array(
                        'manageattachments',
                    ),
                ),
            )),
        );
    }

    public function revert_data()
    {
        return array(
            array('config.remove', array('version_manageattachments')),

            // Delete ACP extension category
            array('module.remove', array(
                'acp',
                'ACP_CAT_DOT_MODS',
                'ACP_MANAGE_ATTACHMENTS',
            )),

            // Delete ACP preferences module
            array('module.remove', array(
                'acp',
                'ACP_MANAGE_ATTACHMENTS',
                array(
                    'module_basename'   => '\pikaron\manageattachments\acp\acp_manageattachments_module',
                    'modes'             => array(
                        'manageattachments',
                    ),
                ),
            )),
        );
    }
}
?>